[PATCH] D106499: [lld][WebAssembly] Align __heap_base

Ayke via Phabricator via llvm-commits llvm-commits at lists.llvm.org
Wed Jul 21 15:49:28 PDT 2021


aykevl updated this revision to Diff 360628.
aykevl edited the summary of this revision.
aykevl added a comment.

- change alignment to 16
- change `heapBaseAlignment` to `heapAlignment`


Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D106499/new/

https://reviews.llvm.org/D106499

Files:
  lld/test/wasm/Inputs/stack-first.s
  lld/test/wasm/stack-first.test
  lld/wasm/Writer.cpp


Index: lld/test/wasm/stack-first.test
===================================================================
--- lld/test/wasm/stack-first.test
+++ lld/test/wasm/stack-first.test
@@ -2,9 +2,10 @@
 ; memory.  In this case the --stack-first option is being passed along with a
 ; stack size of 512.  This means (since the stack grows down) the stack pointer
 ; global should be initialized to 512.
+; Also test that __heap_base is still aligned with the --stack-first option.
 
-RUN: llvm-mc -filetype=obj -triple=wasm32-unknown-unknown %p/Inputs/start.s -o %t.o
-RUN: wasm-ld -z stack-size=512 --stack-first --export=__data_end --export=__heap_base -o %t.wasm %t.o
+RUN: llvm-mc -filetype=obj -triple=wasm32-unknown-unknown %p/Inputs/stack-first.s -o %t.o
+RUN: wasm-ld -z stack-size=512 --stack-first --export=__data_end --export=__heap_base --export=someByte -o %t.wasm %t.o
 RUN: obj2yaml %t.wasm | FileCheck %s
 
 CHECK:        - Type:            GLOBAL
@@ -26,7 +27,13 @@
 CHECK-NEXT:         Mutable:         false
 CHECK-NEXT:         InitExpr:        
 CHECK-NEXT:           Opcode:          I32_CONST
-CHECK-NEXT:           Value:           512
+CHECK-NEXT:           Value:           513
+CHECK-NEXT:       - Index:           3
+CHECK-NEXT:         Type:            I32
+CHECK-NEXT:         Mutable:         false
+CHECK-NEXT:         InitExpr:        
+CHECK-NEXT:           Opcode:          I32_CONST
+CHECK-NEXT:           Value:           528
 CHECK-NEXT:   - Type:            EXPORT
 CHECK-NEXT:     Exports:         
 CHECK-NEXT:       - Name:            memory
@@ -35,9 +42,12 @@
 CHECK-NEXT:       - Name:            _start
 CHECK-NEXT:         Kind:            FUNCTION
 CHECK-NEXT:         Index:           0
-CHECK-NEXT:       - Name:            __data_end
+CHECK-NEXT:       - Name:            someByte
 CHECK-NEXT:         Kind:            GLOBAL
 CHECK-NEXT:         Index:           1
-CHECK-NEXT:       - Name:            __heap_base
+CHECK-NEXT:       - Name:            __data_end
 CHECK-NEXT:         Kind:            GLOBAL
 CHECK-NEXT:         Index:           2
+CHECK-NEXT:       - Name:            __heap_base
+CHECK-NEXT:         Kind:            GLOBAL
+CHECK-NEXT:         Index:           3
Index: lld/test/wasm/Inputs/stack-first.s
===================================================================
--- /dev/null
+++ lld/test/wasm/Inputs/stack-first.s
@@ -0,0 +1,11 @@
+  .globl  _start
+_start:
+  .functype _start () -> ()
+  end_function
+
+  .globl     someByte
+  .type      someByte, at object
+  .section  .data.someByte,"",@
+someByte:
+  .int8 42
+  .size someByte, 1


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D106499.360628.patch
Type: text/x-patch
Size: 3635 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20210721/b425b25c/attachment.bin>


More information about the llvm-commits mailing list
Index: lld/wasm/Writer.cpp
===================================================================
--- lld/wasm/Writer.cpp
+++ lld/wasm/Writer.cpp
@@ -43,6 +43,7 @@
 namespace lld {
 namespace wasm {
 static constexpr int stackAlignment = 16;
+static constexpr int heapAlignment = 16;
 
 namespace {
 
@@ -309,9 +310,12 @@
     placeStack();
 
   if (WasmSym::heapBase) {
-    // Set `__heap_base` to directly follow the end of the stack or global data.
-    // The fact that this comes last means that a malloc/brk implementation
-    // can grow the heap at runtime.
+    // Set `__heap_base` to follow the end of the stack or global data. The
+    // fact that this comes last means that a malloc/brk implementation can
+    // grow the heap at runtime.
+    // We'll align the heap base here because memory allocators might expect
+    // __heap_base to be aligned already.
+    memoryPtr = alignTo(memoryPtr, heapAlignment);
     log("mem: heap base   = " + Twine(memoryPtr));
     WasmSym::heapBase->setVA(memoryPtr);
   }